MAIN_HOST = "https://sky.brunt.co"
MAIN_THINGS_PATH = {"path": "/thing", "host": MAIN_HOST}
THINGS_HOST = "https://thing.brunt.co:8080"
REQUEST_POSITION_KEY = "requestPosition"
DT_FORMAT_STRING = r"%a, %d-%b-%Y %H:%M:%S %Z"
COOKIE_DOMAIN = "brunt.co"